@@ -14,24 +14,19 @@
 """Mongo Persistent Data Manager"""
 from __future__ import absolute_import
 import UserDict
+import bson
 import logging
-import persistent
-import pymongo
-import pymongo.dbref
 import transaction
-import traceback
 import sys
 import zope.interface
 
 from zope.exceptions import exceptionformatter
-from mongopersist import interfaces, serialize
+from mongopersist import conflict, interfaces, serialize
 
+MONGO_ACCESS_LOGGING = False
 COLLECTION_LOG = logging.getLogger('mongopersist.collection')
 
-def create_conflict_error(obj, new_doc):
-    return interfaces.ConflictError(
-        None, obj,
-        (new_doc.get('_py_serial', 0), serialize.u64(obj._p_serial)))
+LOG = logging.getLogger(__name__)
 
 def process_spec(collection, spec):
     try:
@@ -76,7 +71,7 @@ class ProcessSpecDecorator(object):
 class LoggingDecorator(object):
 
     # these are here to be easily patched
-    ADDTB = True
+    ADD_TB = True
     TB_LIMIT = 10 # 10 should be sufficient to figure
 
     def __init__(self, collection, function):
@@ -84,7 +79,7 @@ class LoggingDecorator(object):
         self.function = function
 
     def __call__(self, *args, **kwargs):
-        if self.ADDTB:
+        if self.ADD_TB:
             try:
                 raise ValueError('boom')
             except:
@@ -93,12 +88,15 @@ class LoggingDecorator(object):
                 tb = ''.join(exceptionformatter.extract_stack(
                     sys.exc_info()[2].tb_frame.f_back, limit=self.TB_LIMIT))
         else:
-            tb = '<omitted>'
+            tb = '  <omitted>'
+
+        txn = transaction.get()
+        txn = '%i - %s' %(id(txn), txn.description),
 
         COLLECTION_LOG.debug(
-            "collection: %s.%s %s,\n args:%r,\n kwargs:%r, \n tb:\n%s",
+            "collection: %s.%s %s,\n TXN:%s,\n args:%r,\n kwargs:%r, \n tb:\n%s",
             self.collection.database.name, self.collection.name,
-            self.function.__name__, args, kwargs, tb)
+            self.function.__name__, txn, args, kwargs, tb)
 
         return self.function(*args, **kwargs)
 
@@ -114,9 +112,26 @@ class CollectionWrapper(object):
         self.__dict__['collection'] = collection
         self.__dict__['_datamanager'] = datamanager
 
+    def find_objects(self, *args, **kw):
+        docs = self.find(*args, **kw)
+        coll = self.collection.name
+        dbname = self.collection.database.name
+        for doc in docs:
+            dbref = bson.dbref.DBRef(coll, doc['_id'], dbname)
+            self._datamanager._latest_states[dbref] = doc
+            yield self._datamanager.load(dbref)
+
+    def find_one_object(self, *args, **kw):
+        doc = self.find_one(*args, **kw)
+        coll = self.collection.name
+        dbname = self.collection.database.name
+        dbref = bson.dbref.DBRef(coll, doc['_id'], dbname)
+        self._datamanager._latest_states[dbref] = doc
+        return self._datamanager.load(dbref)
+
     def __getattr__(self, name):
         attr = getattr(self.collection, name)
-        if name in self.LOGGED_METHODS:
+        if MONGO_ACCESS_LOGGING  and name in self.LOGGED_METHODS:
             attr = LoggingDecorator(self.collection, attr)
         if name in self.QUERY_METHODS:
             attr = FlushDecorator(self._datamanager, attr)
@@ -172,33 +187,37 @@ class Root(UserDict.DictMixin):
 class MongoDataManager(object):
     zope.interface.implements(interfaces.IMongoDataManager)
 
-    detect_conflicts = False
     default_database = 'mongopersist'
     name_map_collection = 'persistence_name_map'
-    conflict_error_factory = staticmethod(create_conflict_error)
+    conflict_handler = None
 
-    def __init__(self, conn, detect_conflicts=None, default_database=None,
+    def __init__(self, conn, default_database=None,
                  root_database=None, root_collection=None,
-                 name_map_collection=None, conflict_error_factory=None):
+                 name_map_collection=None,
+                 conflict_handler_factory=conflict.NoCheckConflictHandler):
         self._conn = conn
         self._reader = serialize.ObjectReader(self)
         self._writer = serialize.ObjectWriter(self)
         self._registered_objects = []
         self._loaded_objects = []
         self._inserted_objects = []
+        self._modified_objects = []
         self._removed_objects = []
+        # Keeps states as found at the beginning of the transaction.
         self._original_states = {}
+        # The latest states written to the database. This is different to the
+        # original states, since changes can be flushed to the database
+        # multiple times per transaction.
+        self._latest_states = {}
         self._needs_to_join = True
         self._object_cache = {}
         self.annotations = {}
-        if detect_conflicts is not None:
-            self.detect_conflicts = detect_conflicts
+        if self.conflict_handler is None:
+            self.conflict_handler = conflict_handler_factory(self)
         if default_database is not None:
             self.default_database = default_database
         if name_map_collection is not None:
             self.name_map_collection = name_map_collection
-        if conflict_error_factory is not None:
-            self.conflict_error_factory = conflict_error_factory
         self.transaction_manager = transaction.manager
         self.root = Root(self, root_database, root_collection)
 
@@ -209,23 +228,6 @@ class MongoDataManager(object):
         db_name, coll_name = self._writer.get_collection_name(obj)
         return self._get_collection(db_name, coll_name)
 
-    def _check_conflicts(self):
-        if not self.detect_conflicts:
-            return
-        # Check each modified object to see whether Mongo has a new version of
-        # the object.
-        for obj in self._registered_objects:
-            # This object is not even added to the database yet, so there
-            # cannot be a conflict.
-            if obj._p_oid is None:
-                continue
-            coll = self._get_collection_from_object(obj)
-            new_doc = coll.find_one(obj._p_oid.id, fields=('_py_serial',))
-            if new_doc is None:
-                continue
-            if new_doc.get('_py_serial', 0) != serialize.u64(obj._p_serial):
-                raise self.conflict_error_factory(obj, new_doc)
-
     def _flush_objects(self):
         # Now write every registered object, but make sure we write each
         # object just once.
@@ -263,7 +265,7 @@ class MongoDataManager(object):
 
     def flush(self):
         # Check for conflicts.
-        self._check_conflicts()
+        self.conflict_handler.check_conflicts(self._registered_objects)
         # Now write every registered object, but make sure we write each
         # object just once.
         self._flush_objects()
@@ -310,6 +312,12 @@ class MongoDataManager(object):
         if self._needs_to_join:
             self.transaction_manager.get().join(self)
             self._needs_to_join = False
+        # If the doc is None, but it has been loaded before, we look it
+        # up. This acts as a great hook for optimizations that load many
+        # documents at once. They can now dump the states into the
+        # _latest_states dictionary.
+        if doc is None:
+            doc = self._latest_states.get(obj._p_oid, None)
         self._reader.set_ghost_state(obj, doc)
         self._loaded_objects.append(obj)
 
@@ -323,8 +331,12 @@ class MongoDataManager(object):
             self.transaction_manager.get().join(self)
             self._needs_to_join = False
 
-        if obj is not None and obj not in self._registered_objects:
-            self._registered_objects.append(obj)
+        if obj is not None:
+            if obj not in self._registered_objects:
+                self._registered_objects.append(obj)
+            if obj not in self._modified_objects:
+                self._modified_objects.append(obj)
+            self.conflict_handler.on_modified(obj)
 
     def abort(self, transaction):
         # Aborting the transaction requires three steps:
@@ -338,13 +350,32 @@ class MongoDataManager(object):
             coll.insert(self._original_states[obj._p_oid])
             del self._original_states[obj._p_oid]
         # 3. Reset any changed states.
-        for db_ref, state in self._original_states.items():
+        for obj in self._modified_objects:
+            db_ref = obj._p_oid
+            __traceback_info__ = (obj, db_ref)
+            state = self._original_states.get(db_ref)
+            if state is None:
+                # This should not happen in a fully running environment, but
+                # the tests abort transactions often without having loaded
+                # objects through proper channels.
+                LOG.warn(
+                    'Original state not found while aborting: %r (%s)',
+                    obj, db_ref.id if db_ref else '')
+                continue
+            if self.conflict_handler.has_conflicts([obj]):
+                # If we have a conflict, we are not going to reset to the
+                # original state. (This is a policy that should be made
+                # pluggable.)
+                LOG.info(
+                    'Conflict detected while aborting: %r (%s)',
+                    obj, db_ref.id if db_ref else '')
+                continue
             coll = self.get_collection(db_ref.database, db_ref.collection)
             coll.update({'_id': db_ref.id}, state, True)
         self.reset()
 
     def commit(self, transaction):
-        self._check_conflicts()
+        self.conflict_handler.check_conflicts(self._registered_objects)
 
     def tpc_begin(self, transaction):
         pass
