@@ -14,32 +14,57 @@
 """Mongo Persistence Zope Containers"""
 import UserDict
 import persistent
-import pymongo.dbref
-import pymongo.objectid
+import bson.dbref
+import bson.objectid
 import zope.component
 from bson.errors import InvalidId
 from rwproperty import getproperty, setproperty
 from zope.container import contained, sample
 from zope.container.interfaces import IContainer
 
-from mongopersist import interfaces, serialize
+from mongopersist import interfaces
 from mongopersist.zope import interfaces as zinterfaces
 
+
 class MongoContained(contained.Contained):
 
+    _v_name = None
+    _m_name_attr = None
+    _m_name_getter = None
+    _m_name_setter = None
+
+    _m_parent_attr = None
+    _m_parent_getter = None
+    _m_parent_setter = None
+    _v_parent = None
+
     @getproperty
     def __name__(self):
-        return getattr(self, '_v_key', None)
+        if self._v_name is None:
+            if self._m_name_attr is not None:
+                self._v_name = getattr(self, self._m_name_attr, None)
+            elif self._m_name_getter is not None:
+                self._v_name = self._m_name_getter()
+        return self._v_name
     @setproperty
     def __name__(self, value):
-        setattr(self, '_v_key', value)
+        if self._m_name_setter is not None:
+            self._m_name_setter(value)
+        self._v_name = value
 
     @getproperty
     def __parent__(self):
-        return getattr(self, '_v_parent', None)
+        if self._v_parent is None:
+            if self._m_parent_attr is not None:
+                self._v_parent = getattr(self, self._m_parent_attr, None)
+            elif self._m_parent_getter is not None:
+                self._v_parent = self._m_parent_getter()
+        return self._v_parent
     @setproperty
     def __parent__(self, value):
-        setattr(self, '_v_parent', value)
+        if self._m_parent_setter is not None:
+            self._m_parent_setter(value)
+        self._v_parent = value
 
 
 class SimpleMongoContainer(sample.SampleContainer, persistent.Persistent):
@@ -61,7 +86,7 @@ class SimpleMongoContainer(sample.SampleContainer, persistent.Persistent):
 
     def __getitem__(self, key):
         obj = super(SimpleMongoContainer, self).__getitem__(key)
-        obj._v_key = key
+        obj._v_name = key
         obj._v_parent = self
         return obj
 
@@ -69,14 +94,14 @@ class SimpleMongoContainer(sample.SampleContainer, persistent.Persistent):
         '''See interface `IReadContainer`'''
         obj = super(SimpleMongoContainer, self).get(key, default)
         if obj is not default:
-            obj._v_key = key
+            obj._v_name = key
             obj._v_parent = self
         return obj
 
     def items(self):
         items = super(SimpleMongoContainer, self).items()
         for key, obj in items:
-            obj._v_key = key
+            obj._v_name = key
             obj._v_parent = self
         return items
 
@@ -160,12 +185,16 @@ class MongoContainer(contained.Contained,
                 filter[key] = value
 
     def _locate(self, obj, doc):
-        obj._v_key = doc[self._m_mapping_key]
-        obj._v_parent = self
+        # Helper method that is only used when locating items that are already
+        # in the container and are simply loaded from Mongo.
+        if obj.__name__ is None:
+            obj.__name__ = doc[self._m_mapping_key]
+        if obj.__parent__ is None:
+            obj._v_parent = self
 
     def _load_one(self, doc):
         # Create a DBRef object and then load the full state of the object.
-        dbref = pymongo.dbref.DBRef(
+        dbref = bson.dbref.DBRef(
             self._m_collection, doc['_id'],
             self._m_database or self._m_jar.default_database)
         # Stick the doc into the _latest_states:
@@ -174,6 +203,12 @@ class MongoContainer(contained.Contained,
         self._locate(obj, doc)
         return obj
 
+    def __cmp__(self, other):
+        # UserDict implements the semantics of implementing comparison of
+        # items to determine equality, which is not what we want for a
+        # container, so we revert back to the default object comparison.
+        return cmp(id(self), id(other))
+
     def __getitem__(self, key):
         filter = self._m_get_items_filter()
         filter[self._m_mapping_key] = key
@@ -183,7 +218,7 @@ class MongoContainer(contained.Contained,
         return obj
 
     def _real_setitem(self, key, value):
-        # This call by iteself caues the state to change _p_changed to True.
+        # This call by iteself causes the state to change _p_changed to True.
         if self._m_mapping_key is not None:
             setattr(value, self._m_mapping_key, key)
         if self._m_parent_key is not None:
@@ -194,7 +229,7 @@ class MongoContainer(contained.Contained,
         # its oid.
         if value._p_oid is None:
             self._m_jar.insert(value)
-        # When the key is None, we use the object is as name.
+        # When the key is None, we use the object id as name.
         if key is None:
             key = unicode(value._p_oid.id)
         # We want to be as close as possible to using the Zope semantics.
@@ -210,9 +245,17 @@ class MongoContainer(contained.Contained,
         value = self[key]
         # First remove the parent and name from the object.
         if self._m_mapping_key is not None:
-            delattr(value, self._m_mapping_key)
+            try:
+                delattr(value, self._m_mapping_key)
+            except AttributeError, err:
+                # Sometimes we do not control those attributes.
+                pass
         if self._m_parent_key is not None:
-            delattr(value, self._m_parent_key)
+            try:
+                delattr(value, self._m_parent_key)
+            except AttributeError, err:
+                # Sometimes we do not control those attributes.
+                pass
         # Let's now remove the object from the database.
         if self._m_remove_documents:
             self._m_jar.remove(value)
@@ -280,12 +323,12 @@ class IdNamesMongoContainer(MongoContainer):
         return True
 
     def _locate(self, obj, doc):
-        obj._v_key = unicode(doc['_id'])
+        obj._v_name = unicode(doc['_id'])
         obj._v_parent = self
 
     def __getitem__(self, key):
         try:
-            id = pymongo.objectid.ObjectId(key)
+            id = bson.objectid.ObjectId(key)
         except InvalidId:
             raise KeyError(key)
         filter = self._m_get_items_filter()
@@ -297,7 +340,7 @@ class IdNamesMongoContainer(MongoContainer):
 
     def __contains__(self, key):
         try:
-            id = pymongo.objectid.ObjectId(key)
+            id = bson.objectid.ObjectId(key)
         except InvalidId:
             return False
         return self.raw_find_one({'_id': id}, fields=()) is not None
