@@ -54,128 +54,196 @@ class Person(container.MongoContained, SimplePerson):
     pass
 
 
-class Campaigns(container.MongoContainer):
+def doctest_MongoContained_simple():
+    """MongoContained: simple use
 
-    _m_collection = 'campaigns'
+    The simplest way to use MongoContained is to use it without any special
+    modification. In this case it is required that the container always sets
+    the name and parent after loading the item. It can do so directly by
+    setting ``_v_name`` and ``_v_parent`` so that the persistence mechanism
+    does not kick in.
 
-    def add(self, campaign):
-        self[campaign.name] = campaign
+      >>> class Simples(container.MongoContainer):
+      ...     def __init__(self, name):
+      ...         super(Simples, self).__init__()
+      ...         self.name = name
+      ...     def __repr__(self):
+      ...         return '<Simples %s>' %self.name
 
+      >>> class Simple(container.MongoContained, persistent.Persistent):
+      ...     pass
 
-class MongoItem(container.MongoContained,
-                persistent.Persistent):
-    pass
+    Let's create a simple component and activate the persistence machinery:
 
-class Campaign(MongoItem, container.MongoContainer):
+      >>> s = Simple()
+      >>> s._p_jar = dm
 
-    _m_collection = 'persons'
-    _p_mongo_collection = 'campaigns'
+    As you can see, the changed flag is not changed:
 
-    def __init__(self, name):
-        self.name = name
-
-    def __repr__(self):
-        return '<%s %s>' %(self.__class__.__name__, self.name)
-
-
-def doctest_Campaign():
-    """MongoContainer: basic
-
-    Let's make sure events are fired correctly:
-
-      >>> zope.component.provideHandler(handleObjectModifiedEvent)
-
-    Let's add a container to the root:
-
-      >>> transaction.commit()
-      >>> dm.root['c'] = Campaigns()
-
-      >>> db = dm._conn[DBNAME]
-      >>> pprint(list(db['campaigns'].find()))
-      []
-
-      [{u'_id': ObjectId('4e7ddf12e138237403000000'),
-        u'_m_collection': u'person'}]
-
-    It is unfortunate that the '_m_collection' attribute is set. This is
-    avoidable using a sub-class.
-
-      >>> dm.root['c'][u'one'] = Campaign(u'one')
-      ContainerModifiedEvent: <...Campaigns ...>
-      >>> dm.root['c'].keys()
-      [u'one']
-      >>> dm.root['c'][u'one']
-      <Campaign one>
-
-      >>> dm.root['c']['one'].__parent__
-      <mongopersist.zope.tests.test_container.Campaigns object at 0x001122>
-      >>> dm.root['c']['one'].__name__
-      u'one'
-
-    It is a feature of the container that the item is immediately available
-    after assignment, but before the data is stored in the database. Let's
-    commit and access the data again:
-
-      >>> transaction.commit()
-
-      >>> pprint(list(db['campaigns'].find()))
-      [{u'_id': ObjectId('4e7ddf12e138237403000000'),
-        u'key': u'one',
-        u'name': u'one',
-        u'parent': DBRef(u'mongopersist.zope.tests.test_container.Campaigns',
-            ObjectId('4e7ddf12e138237403000000'),
-            u'mongopersist_container_test')}]
-
-      >>> 'one' in dm.root['c']
-      True
-      >>> dm.root['c'].keys()
-      [u'one']
-      >>> dm.root['c']['one'].__parent__
-      <mongopersist.zope.tests.test_container.Campaigns object at 0x001122>
-      >>> dm.root['c']['one'].__name__
-      u'one'
-
-    We get a usual key error, if an object does not exist:
-
-      >>> dm.root['c']['roy']
-      Traceback (most recent call last):
-      ...
-      KeyError: 'roy'
-
-      >>> 'roy' in dm.root['c']
+      >>> s._p_changed
+      False
+      >>> s._v_name = 'simple'
+      >>> s._v_parent = Simples('one')
+      >>> s._p_changed
       False
 
-    Now remove the item:
+    And accessing the name and parent works:
 
-      >>> del dm.root['c']['one']
-      ContainerModifiedEvent: <...Campaigns ...>
+      >>> s.__name__
+      'simple'
+      >>> s.__parent__
+      <Simples one>
 
-    The changes are immediately visible.
+    But assignment works as well.
 
-      >>> dm.root['c'].keys()
-      []
-      >>> dm.root['c']['one']
-      Traceback (most recent call last):
-      ...
-      KeyError: 'one'
+      >>> s.__name__ = 'simple2'
+      >>> s.__name__
+      'simple2'
+      >>> s.__parent__ = Simples('two')
+      >>> s.__parent__
+      <Simples two>
+      >>> s._p_changed
+      True
+    """
 
-    Make sure it is really gone after committing:
+def doctest_MongoContained_proxy_attr():
+    """MongoContained: proxy attributes
 
-      >>> transaction.commit()
-      >>> dm.root['c'].keys()
-      []
+    It is also possible to use proxy attributes to reference the name and
+    parent. This allows you to have nice attribute names for storage in Mongo.
 
-    Check adding of more objects:
+    The main benefit, though is the ability of the object to load its
+    location, so that you can load the object without going through the
+    container and get full location path.
 
-      >>> dm.root['c'][u'1'] = c1 = Campaign(u'One')
-      ContainerModifiedEvent: <...Campaigns ...>
-      >>> dm.root['c'][u'2'] = c2 = Campaign(u'Two')
-      ContainerModifiedEvent: <...Campaigns ...>
-      >>> dm.root['c'][u'3'] = Campaign(u'Three')
-      ContainerModifiedEvent: <...Campaigns ...>
+      >>> class Proxies(container.MongoContainer):
+      ...     def __init__(self, name):
+      ...         super(Proxies, self).__init__()
+      ...         self.name = name
+      ...     def __repr__(self):
+      ...         return '<Proxies %s>' %self.name
 
-      >>> sorted(dm.root['c'].keys())
-      [u'1', u'2', u'3']
+      >>> class Proxy(container.MongoContained, persistent.Persistent):
+      ...     _m_name_attr = 'name'
+      ...     _m_parent_attr = 'parent'
+      ...     def __init__(self, name, parent):
+      ...         self.name = name
+      ...         self.parent = parent
+
+    Let's create a proxy component and activate the persistence machinery:
+
+      >>> p = Proxy('proxy', Proxies('one'))
+      >>> p._p_jar = dm
+
+    So accessing the name and parent works:
+
+      >>> p.__name__
+      'proxy'
+      >>> p.__parent__
+      <Proxies one>
+
+    But assignment is only stored into the volatile variables and the proxy
+    attribute values are not touched.
+
+      >>> p.__name__ = 'proxy2'
+      >>> p.__name__
+      'proxy2'
+      >>> p.name
+      'proxy'
+      >>> p.__parent__ = Proxies('two')
+      >>> p.__parent__
+      <Proxies two>
+      >>> p.parent
+      <Proxies one>
+
+    This behavior is intentional, so that containment machinery cannot mess
+    with the real attributes. Note that in practice, only MongoContainer sets
+    the ``__name__`` and ``__parent__`` and it should be always consistent
+    with the referenced attributes.
+
+    """
+
+def doctest_MongoContained_setter_getter():
+    """MongoContained: setter/getter functions
+
+    If you need ultimate flexibility of where to get and store the name and
+    parent, then you can define setters and getters.
+
+      >>> class Funcs(container.MongoContainer):
+      ...     def __init__(self, name):
+      ...         super(Funcs, self).__init__()
+      ...         self.name = name
+      ...     def __repr__(self):
+      ...         return '<Funcs %s>' %self.name
+
+      >>> class Func(container.MongoContained, persistent.Persistent):
+      ...     _m_name_getter = lambda s: s.name
+      ...     _m_name_setter = lambda s, v: setattr(s, 'name', v)
+      ...     _m_parent_getter = lambda s: s.parent
+      ...     _m_parent_setter = lambda s, v: setattr(s, 'parent', v)
+      ...     def __init__(self, name, parent):
+      ...         self.name = name
+      ...         self.parent = parent
+
+    Let's create a func component and activate the persistence machinery:
+
+      >>> f = Func('func', Funcs('one'))
+      >>> f._p_jar = dm
+
+    So accessing the name and parent works:
+
+      >>> f.__name__
+      'func'
+      >>> f.__parent__
+      <Funcs one>
+
+    In this case, the setters are used, if the name and parent are changed:
+
+      >>> f.__name__ = 'func2'
+      >>> f.__name__
+      'func2'
+      >>> f.name
+      'func2'
+      >>> f.__parent__ = Funcs('two')
+      >>> f.__parent__
+      <Funcs two>
+      >>> f.parent
+      <Funcs two>
+    """
+
+
+def doctest_MongoContained_mixed():
+    """MongoContained: mixed usage
+
+    When the container is stored in the ZODB or another persistence mechanism,
+    a mixed usage of proxy attributes and getter/setter functions is the best
+    appraoch.
+
+      >>> class Mixers(btree.BTreeContainer):
+      ...     def __init__(self, name):
+      ...         super(Mixers, self).__init__()
+      ...         self.name = name
+      ...     def __repr__(self):
+      ...         return '<Mixers %s>' %self.name
+      >>> mixers = Mixers('one')
+
+      >>> class Mixer(container.MongoContained, persistent.Persistent):
+      ...     _m_name_attr = 'name'
+      ...     _m_parent_getter = lambda s: mixers
+      ...     def __init__(self, name):
+      ...         self.name = name
+
+    Let's create a mixer component and activate the persistence machinery:
+
+      >>> m = Mixer('mixer')
+      >>> m._p_jar = dm
+
+    So accessing the name and parent works:
+
+      >>> m.__name__
+      'mixer'
+      >>> m.__parent__
+      <Mixers one>
     """
 
 
@@ -832,6 +900,168 @@ def doctest_MongoContainer_with_ZODB():
     Note that we produced a nice hex-presentation of the ZODB's OID.
     """
 
+
+# classes for doctest_Realworldish
+class Campaigns(container.MongoContainer):
+    _m_collection = 'campaigns'
+
+    def __init__(self, name):
+        self.name = name
+
+    def add(self, campaign):
+        self[campaign.name] = campaign
+
+    def __repr__(self):
+        return '<%s %s>' % (self.__class__.__name__, self.name)
+
+
+class MongoItem(container.MongoContained,
+                persistent.Persistent):
+    pass
+
+
+class Campaign(MongoItem, container.MongoContainer):
+    _m_collection = 'persons'
+    _p_mongo_collection = 'campaigns'
+
+    def __init__(self, name):
+        self.name = name
+
+    def __repr__(self):
+        return '<%s %s>' % (self.__class__.__name__, self.name)
+
+
+class PersonItem(MongoItem):
+    _p_mongo_collection = 'persons'
+
+    def __init__(self, name):
+        self.name = name
+
+    def __str__(self):
+        return self.name
+
+    def __repr__(self):
+        return '<%s %s>' % (self.__class__.__name__, self)
+
+
+def doctest_Realworldish():
+    """Let's see some real worldish hierarchic structure persisted
+
+    Let's make sure events are fired correctly:
+
+      >>> zope.component.provideHandler(handleObjectModifiedEvent)
+
+    Let's add a container to the root:
+
+      >>> transaction.commit()
+      >>> dm.root['c'] = Campaigns('foobar')
+
+      >>> db = dm._conn[DBNAME]
+      >>> pprint(list(db['mongopersist.zope.tests.test_container.Campaigns'].find()))
+      [{u'_id': ObjectId('4e7ddf12e138237403000000'),
+        u'name': u'foobar'}]
+
+    It is unfortunate that the '_m_collection' attribute is set. This is
+    avoidable using a sub-class.
+
+      >>> dm.root['c'][u'one'] = Campaign(u'one')
+      ContainerModifiedEvent: <...Campaigns ...>
+      >>> dm.root['c'].keys()
+      [u'one']
+      >>> dm.root['c'][u'one']
+      <Campaign one>
+
+      >>> dm.root['c']['one'].__parent__
+      <Campaigns foobar>
+      >>> dm.root['c']['one'].__name__
+      u'one'
+
+    It is a feature of the container that the item is immediately available
+    after assignment, but before the data is stored in the database. Let's
+    commit and access the data again:
+
+      >>> transaction.commit()
+
+      >>> pprint(list(db['campaigns'].find()))
+      [{u'_id': ObjectId('4e7ddf12e138237403000000'),
+        u'key': u'one',
+        u'name': u'one',
+        u'parent': DBRef(u'mongopersist.zope.tests.test_container.Campaigns',
+            ObjectId('4e7ddf12e138237403000000'),
+            u'mongopersist_container_test')}]
+
+      >>> 'one' in dm.root['c']
+      True
+      >>> dm.root['c'].keys()
+      [u'one']
+      >>> dm.root['c']['one'].__parent__
+      <Campaigns foobar>
+      >>> dm.root['c']['one'].__name__
+      u'one'
+
+    We get a usual key error, if an object does not exist:
+
+      >>> dm.root['c']['roy']
+      Traceback (most recent call last):
+      ...
+      KeyError: 'roy'
+
+      >>> 'roy' in dm.root['c']
+      False
+
+    Now remove the item:
+
+      >>> del dm.root['c']['one']
+      ContainerModifiedEvent: <...Campaigns ...>
+
+    The changes are immediately visible.
+
+      >>> dm.root['c'].keys()
+      []
+      >>> dm.root['c']['one']
+      Traceback (most recent call last):
+      ...
+      KeyError: 'one'
+
+    Make sure it is really gone after committing:
+
+      >>> transaction.commit()
+      >>> dm.root['c'].keys()
+      []
+
+    Check adding of more objects:
+
+      >>> dm.root['c'][u'1'] = c1 = Campaign(u'One')
+      ContainerModifiedEvent: <...Campaigns ...>
+      >>> dm.root['c'][u'2'] = c2 = Campaign(u'Two')
+      ContainerModifiedEvent: <...Campaigns ...>
+      >>> dm.root['c'][u'3'] = Campaign(u'Three')
+      ContainerModifiedEvent: <...Campaigns ...>
+
+      >>> sorted(dm.root['c'].keys())
+      [u'1', u'2', u'3']
+
+    Check adding of more subitems:
+
+      >>> stephan = c1['stephan'] = PersonItem('Stephan')
+      ContainerModifiedEvent: <Campaign One>
+      >>> roy = c1['roy'] = PersonItem('Roy')
+      ContainerModifiedEvent: <Campaign One>
+
+      >>> sorted(c1.keys())
+      [u'roy', u'stephan']
+
+      >>> adam = c2['adam'] = PersonItem('Adam')
+      ContainerModifiedEvent: <Campaign Two>
+
+      >>> sorted(c1.keys())
+      [u'roy', u'stephan']
+      >>> sorted(c2.keys())
+      [u'adam']
+
+    """
+
+
 checker = renormalizing.RENormalizing([
     (re.compile(r'datetime.datetime(.*)'),
      'datetime.datetime(2011, 10, 1, 9, 45)'),
